#ifndef INTERFACE_H
#define INTERFACE_H

#include <QTimer>
#include <QImage>
#include "../common/sources/system/cmaths.h"
#include "character.h"
#include "mouse.h"
#include <map>
#include <string>

#define MAIN_INTERFACE_COLOR    QColor(0, 204, 204)
#define DEFAULT_TEXT_COLOR      QColor(173, 173, 173)
#define HIGHLIGHT_TEXT_COLOR    QColor(255, 170, 0)
#define INVENTORY_BG_COLOR      QColor(68, 68, 68)
#define INVENTORY_SLOT_COLOR    QColor(102, 102, 102)
#define OVERLOAD1_COLOR         QColor(255, 255, 0)
#define OVERLOAD2_COLOR         QColor(255, 0, 0)

#define CHAMP_PORTRAIT_WIDTH    32
#define CHAMP_PORTRAIT_HEIGHT   29
#define ITEM_WIDTH    16
#define ITEM_HEIGHT   16

#define OVERLOAD0_DELAY         1
#define OVERLOAD1_DELAY         25
#define OVERLOAD2_DELAY         40

#define ARROWS_HIGHLIGHT_TIME   20

class CInterface
{
public:
    // Main state of the interface
    enum EMainState
    {
        eMainGame = 0,  // standard window with the 3D view
        eMainInventory  // Inventory sheet
    };

    // States of the weapon's area
    enum EWeaponsAreaState
    {
        eWeaponsAreaWeapons = 0,    // The 4 weapons boxes
        eWeaponsAreaAttacks,        // The attacks of a weapon
        eWeaponsAreaDamage          // The damages when we hit a monster
    };

    // fonts
    enum EFonts
    {
        eFontStandard = 0,
        eFontScroll,
        eFontWalls
    };

    enum EArrows
    {
        eArrowForward = 0,
        eArrowBackward,
        eArrowLeft,
        eArrowRight,
        eArrowTurnLeft,
        eArrowTurnRight,
        eArrowsCount
    };

    struct SBodyInfos
    {
        CVec2   pos;
        char    type;
        int     defaultImage;
    };

    CInterface();
    ~CInterface();

    void    display(QImage* image);
    void    update(SMouseArea *clickedArea);
    bool    isInInventory();
    CRect   getChampionPortraitRect(int num);
    CRect   getItemRect(int num);
    void    drawText(QImage* image, CVec2 pos, EFonts font, const char* text, QColor color);
    void    readTextDB();
    std::string getText(std::string id);
    void    enterResurrect(int num);
    bool    isChampionEmpty(int num);

    int     selectedChampion;

private:
    bool    isChampionDead(int num);

    bool    isWeaponEmpty(int num);
    bool    isWeaponGreyed(int num);

    bool    isArrowsGreyed();
    bool    isSpellsGreyed();

    void        drawBar(QImage* image, int num, CVec2 pos, CCharacter::EStatsNames statName);
    void        drawChampionName(QImage* image, int num, CVec2 pos, bool isFullName);
    void        drawChampion(QImage* image, int num);
    void        drawObjectName(QImage* image);
    void        drawSpells(QImage* image);
    void        drawWeaponsArea(QImage* image);
    void        drawWeapon(QImage* image, int num);
    void        drawArrows(QImage* image);
    void        drawInventory(QImage* image);
    void        drawInfos(QImage* image);
    CObject&    getWeapon(int num);
    void        drawBodyPart(QImage* image, CVec2 pos, int championNum, CCharacter::EBodyParts part, bool enableArea = true);
    CCharacter::SStat   getStat(int num, CCharacter::EStatsNames statName);
    void        displayStat(QImage* image, CVec2 pos, int x2, CCharacter::EStatsNames num);
    void        updateArrows(SMouseArea* clickedArea);
    bool        isClickingOnArrow(SMouseArea* clickedArea, EMouseAreaTypes type);
    void        setDelayedArrow(EArrows arrow);

    EMainState          mainState;
    EWeaponsAreaState   weaponsAreaState;
    int     currentChampion;
    int     currentWeapon;
    QTimer* weaponCoolDown[4];
    std::map<std::string, std::string>  textsDB;
    bool    isResurrecting;
    bool    isPressingEye;
    EArrows currentArrow;
    int     arrowHighlightTime;
    CRect   arrowsRects[CInterface::eArrowsCount];
    SBodyInfos  bodyInfos[CCharacter::eBodyPartCount];
    EArrows delayedArrow;
    int     delayedArrowTime;
};

extern CInterface   interface;

#endif // INTERFACE_H
