#include "player.h"
#include "interface.h"
#include "tiles.h"
#include "doors.h"

CPlayer  player;

//-----------------------------------------------------------------------------------------
CPlayer::CPlayer()
{
    dir = 0;
}

//-----------------------------------------------------------------------------------------
CVec2   CPlayer::getPosFromLocal(CVec2 localPos)
{
    CVec2   newLocalPos;

    switch (dir)
    {
        case 0: // up
            newLocalPos = localPos;
            break;

        case 1: // left
            newLocalPos.x = localPos.y;
            newLocalPos.y = -localPos.x;
            break;

        case 2: // down
            newLocalPos.x = -localPos.x;
            newLocalPos.y = -localPos.y;
            break;

        case 3: // right
            newLocalPos.x = -localPos.y;
            newLocalPos.y = localPos.x;
            break;
    }

    return pos + newLocalPos;
}

//-----------------------------------------------------------------------------------------
EWallSide   CPlayer::getWallSide(EWallSide side)
{
    static const EWallSide rotatedSides[4][4] =
    {
        {eWallSideUp,    eWallSideDown,  eWallSideLeft,  eWallSideRight}, // up
        {eWallSideLeft,  eWallSideRight, eWallSideDown,  eWallSideUp},    // left
        {eWallSideDown,  eWallSideUp,    eWallSideRight, eWallSideLeft},  // down
        {eWallSideRight, eWallSideLeft,  eWallSideUp,    eWallSideDown}   // right
    };
    return rotatedSides[dir][side];
}

//-----------------------------------------------------------------------------------------
void        CPlayer::turnLeft()
{
    if (interface.isInInventory() == false)
        dir = (dir + 1) % 4;
}

//-----------------------------------------------------------------------------------------
void        CPlayer::turnRight()
{
    if (interface.isInInventory() == false)
        dir = (dir + 4 - 1) % 4;
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveLeft()
{
    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideLeft);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveRight()
{
    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideRight);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveForward()
{
    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideUp);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void        CPlayer::moveBackward()
{
    if (interface.isInInventory() == false)
    {
        EWallSide   side = getWallSide(eWallSideDown);
        moveIfPossible(side);
    }
}

//-----------------------------------------------------------------------------------------
void CPlayer::moveIfPossible(EWallSide side)
{
    CVec2   newPos = pos;

    switch (side)
    {
    case eWallSideUp:
        newPos.y--;
        break;

    case eWallSideDown:
        newPos.y++;
        break;

    case eWallSideLeft:
        newPos.x--;
        break;

    case eWallSideRight:
        newPos.x++;
        break;

    default:
        break;
    }

    CTile*  tile = map.getTile(pos);
    CTile*  destTile = map.getTile(newPos);

    bool canMove = true;

    if (tile->mWalls[side].getType() != 0)
    {
        canMove = false;
    }
    else if (destTile == NULL)
    {
        canMove = false;
    }
    else if (destTile->getType() == eTileDoor && doors.isOpened(destTile) == false)
    {
        canMove = false;
    }

    if (canMove == true)
    {
        pos = newPos;
    }
}
